Observe the difference in behaviour by executing command
`org-src-fontify-block' on source blocks below.
#+BEGIN_SRC emacs-lisp :results silent
  (save-excursion
    (while (ignore-errors (org-babel-next-src-block) t)
      (org-src-fontify-block)))
#+END_SRC

This is what correctly fontified block should look like:
#+BEGIN_SRC diff
diff --git a/lisp/org-src.el b/lisp/org-src.el
index 6e01b8a..8ce4a8b 100644
--- a/lisp/org-src.el
+++ b/lisp/org-src.el
@@ -922,6 +922,7 @@ fontification of code blocks see `org-src-fontify-block' and
              (get-buffer-create
               (concat " org-src-fontification:" (symbol-name lang-mode)))
            (delete-region (point-min) (point-max))
            (insert string " ") ;; so there's a final property change
+           (unless org-src-preserve-indentation (org-do-remove-indentation))
            (unless (eq major-mode lang-mode) (funcall lang-mode))
            (font-lock-fontify-buffer)

#+END_SRC

Fontification on this block will fail because of present indentation:
#+BEGIN_SRC diff
  diff --git a/lisp/org-src.el b/lisp/org-src.el
  index 6e01b8a..8ce4a8b 100644
  --- a/lisp/org-src.el
  +++ b/lisp/org-src.el
  @@ -922,6 +922,7 @@ fontification of code blocks see `org-src-fontify-block' and
                (get-buffer-create
                 (concat " org-src-fontification:" (symbol-name lang-mode)))
              (delete-region (point-min) (point-max))
              (insert string " ") ;; so there's a final property change
  +           (unless org-src-preserve-indentation (org-do-remove-indentation))
              (unless (eq major-mode lang-mode) (funcall lang-mode))
              (font-lock-fontify-buffer)
#+END_SRC

You will be able to observe correct fontification on both blocks by
applying new definition of function `org-src-font-lock-fontify-block'
produced by my patch:

#+BEGIN_SRC emacs-lisp :results silent
  (defun org-src-font-lock-fontify-block (lang start end)
    "Fontify code block.
  This function is called by emacs automatic fontification, as long
  as `org-src-fontify-natively' is non-nil.  For manual
  fontification of code blocks see `org-src-fontify-block' and
  `org-src-fontify-buffer'"
    (let ((lang-mode (org-src-get-lang-mode lang)))
      (if (fboundp lang-mode)
          (let ((string (buffer-substring-no-properties start end))
                (modified (buffer-modified-p))
                (org-buffer (current-buffer)) pos next)
            (remove-text-properties start end '(face nil))
            (with-current-buffer
                (get-buffer-create
                 (concat " org-src-fontification:" (symbol-name lang-mode)))
              (delete-region (point-min) (point-max))
              (insert string " ") ;; so there's a final property change
              (unless org-src-preserve-indentation (org-do-remove-indentation))
              (unless (eq major-mode lang-mode) (funcall lang-mode))
              (font-lock-fontify-buffer)
              (setq pos (point-min))
              (while (setq next (next-single-property-change pos 'face))
                (unless org-src-preserve-indentation
                  (let ((eol (save-excursion (goto-char pos)
                                             (line-end-position))))
                    (with-current-buffer org-buffer
                      (goto-char (+ start (1- pos)))
                      (setq start (- (line-end-position) (1- eol))))
                    (when (< eol next)
                      (setq next (1+ eol)))))
                (put-text-property
                 (+ start (1- pos)) (1- (+ start next)) 'face
                 (get-text-property pos 'face) org-buffer)
                (setq pos next)))
            (add-text-properties
             start end
             '(font-lock-fontified t fontified t font-lock-multiline t))
            (set-buffer-modified-p modified)))))
#+END_SRC
